package K::Web;

use strict;
use IO::Socket::INET;
use Carp qw(croak);
use URI::Escape;
use MIME::Base64 qw(encode_base64);

eval 'use IO::Socket::SSL';
my $ssl = 1 unless $@;

our $VERSION = 0.2;

sub new
{
	my ($self, %cfg) = @_;
	
	my $obj = bless
	{
		agent => $cfg{agent} || 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2)',
		timeout => $cfg{timeout} || 20,
		verbose => $cfg{verbose} || 0,
	}, $self;
	
	return $obj;
}

sub set_cookie
{
	my ($self, $cookie) = @_;
	
	$self->{cookie} = $cookie;
}

sub set_cookie_ex
{
	my ($self, $response) = @_;
	
	$self->{cookie} = join ' ', ($response =~ /Set-Cookie: (.+?;|\r)/igs);
}

sub header
{
	my ($self, %headers) = @_;
	
	$self->{headers} .= "$_\r\n" for map("$_: $headers{$_}", keys %headers);
}

sub proxy_auth
{
	my ($self, $login, $passw) = @_;
	
	$self->{p_auth} = encode_base64 $login.':'.$passw, '';
}

sub socks_auth
{
	my ($self, $login, $passw) = @_;
	
	my $l_len = chr length $login;
	my $p_len = chr length $passw;
	$self->{s_auth} = pack('H*','01').$l_len.$login.$p_len.$passw;
}

sub proxy
{
	my ($self, $proxy_type, $proxy) = @_;

	($self->{proxy_host}, $self->{proxy_port}) = split /:/, $proxy;
	$self->{proxy} = $proxy_type && $self->{proxy_host} && $self->{proxy_port} ? $proxy_type : 0;
}

sub request
{
	my ($self, $method, $host, $port, $uri, $encode, $payload, $limit) = 
	(
		@_[0..2],
		$_[3] || 80,	#port
		$_[4] || '/',	#uri
		$_[5] || 1, 	#encode
		$_[6] || '', 	#payload
		$_[7] || 0 		#limit
	);
	
	
	
	my $scheme;
	($scheme, $host) = ($host =~ m{^(https*)://(.+)$});

	my $header =
	"$method $uri HTTP/1.0\r\n".
	"Host: $host\r\n".
	"User-Agent: $self->{agent}\r\n".
	($self->{p_auth} ? "Proxy-Authorization: Basic $self->{p_auth}\r\n" : "").
	($self->{cookie} ? "Cookie: $self->{cookie}\r\n" : "").
	($self->{headers} ? $self->{headers} : "").
	"Connection: close\r\n";

	if($method eq 'GET')
	{
		$header .= "\r\n";
	}
	elsif(defined $payload && $method eq 'POST')
	{
		if(ref $payload eq 'HASH')
		{
			my $data;
			$header .= "Content-Type: multipart/form-data; boundary=2011\r\n";
			for my $key(keys %{$payload}) #fuu...
			{
				$key = uri_escape $key if $encode;
				if(ref $payload->{$key} eq 'ARRAY')
				{
					$data .=
					"--2011\r\n".
					"Content-Disposition: form-data; name=\"$key\"; filename=\"".(@{$payload->{$key}}[0] =~ m{^(?:.*/|)(.+)$})[0]."\"\r\n".
					"Content-Type: application/octetstream\r\n\r\n".
					 _load_bin(@{$payload->{$key}}[0])."\r\n";
				}
				else
				{
					$data .= 
					"--2011\r\n".
					"Content-Disposition: form-data; name=\"$key\"\r\n\r\n".
					($encode ? uri_escape $payload->{$key} : $payload->{$key}).
					"\r\n";
				}
			}
			$payload = $data."--2011--";
		}
		else
		{
			$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
		}
		$header .= "Content-Length: ".length($payload)."\r\n\r\n".$payload;
	}

	$self->{host} = $host;
	$self->{port} = $port;

	my $socket;
	
	croak 'No SSL support' if (!$ssl && $scheme eq 'https');

	if($scheme eq 'https' && !$self->{proxy})
	{
		$socket = IO::Socket::SSL->new
		(
			PeerAddr => $self->{host},
			PeerPort => $self->{port},
			PeerProto => 'tcp', 
			TimeOut => $self->{timeout},
		);
	}
	elsif($scheme eq 'http' || $scheme eq 'https')
	{
		$socket = IO::Socket::INET->new
		(
			PeerAddr => $self->{proxy} ? $self->{proxy_host} : $self->{host},
			PeerPort => $self->{proxy} ? $self->{proxy_port} : $self->{port},
			PeerProto => 'tcp', 
			TimeOut => $self->{timeout},
		);
	}
	else
	{
		croak "Unsupported scheme: $scheme";
	}

	#re
	unless($socket)
	{
		warn "Can't connect to ".($self->{proxy} ? $self->{proxy_host} : $self->{host})."\n" if $self->{verbose};
		return '';
	}
	elsif($socket && $scheme eq 'https' && $self->{proxy} == 1)
	{
		syswrite $socket, "CONNECT $host:$port HTTP/1.0\r\n\r\n";
		
		my $answ = '';
		while(sysread $socket, my $buf, 64)
		{
			$answ .= $buf;
			last if $buf =~ /\r\n\r\n/; #erroneous implementation
		}

		if($answ =~ /Connection Established/)
		{
			IO::Socket::SSL->start_SSL($socket);
		}
		else
		{
			close $socket;
			return '';
		}
	}
	elsif($socket && $self->{proxy} == 2)
	{
		if(_socks($self, $socket))
		{
			IO::Socket::SSL->start_SSL($socket) if $scheme eq 'https';
		}
		else
		{
			close $socket;
			return '';
		}
	}
	

	syswrite $socket, $header;
	my $answ = '';

	while(sysread $socket, my $buf, 32)
	{
		$answ .= $buf;
		last if $limit && length $answ >= $limit;
	}
	
	close $socket;
	
	return $answ;
}

sub _socks
{
	my ($self, $socket) = @_;
	
	my $hex = pack 'H*', '05020002';
    syswrite $socket, $hex;
	sysread $socket, my $result, 4;

	$result = unpack 'H*', $result;
	
	if($result == '0502' && $self->{s_auth})
	{
		syswrite $socket, $self->{s_auth};
		sysread $socket, $result, 4;
		$result = substr unpack('H*', $result), 3, 1;
		return undef if $result;
	}
	elsif($result != '0500')
	{
		return undef;
	}
	
    my $len_h = chr length $self->{host};
    $hex = pack('H*','05010003').$len_h.$self->{host}.pack('n',$self->{port});
	
    syswrite $socket, $hex;
	sysread $socket, $result, 100;
	
	$result = substr unpack('H*', $result), 3, 1;
	
	return $result ? undef : 1;
}

sub _load_bin
{
	my ($file) = @_;
	
	open F, '<', $file or warn "Can't open file $file - $!\n";
	binmode F;
	undef local $/;
	$file = <F>;
	close F;
	
	return $file;
}

1;